/*
 * resolv_manager.c
 * This file is part of Network-inador
 *
 * Copyright (C) 2022 - Félix Arreola Rodríguez
 *
 * Network-inador is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Network-inador is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Network-inador; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, 
 * Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <ctype.h>
#include <sys/inotify.h>

#include "common.h"
#include "utils.h"
#include "resolv_conf_parser.h"

void resolv_manager_clear_entries_by_prog (NetworkInadorHandle *handle, const int iface_index, const char *prog) {
	GList *g, *next;
	ResolvConfEntry *entry;
	
	g = handle->resolver_entries;
	while (g != NULL) {
		next = g->next;
		
		entry = (ResolvConfEntry *) g->data;
		
		if (entry->origin != RESOLV_ORIGIN_RESOLVCONF) {
			/* Solo borramos las entradas creadas via resolvconf */
			g = next;
			continue;
		}
		
		if (entry->owner_interface_index != iface_index) {
			g = next;
			continue;
		}
		
		if (prog == NULL || strcmp (entry->owner_prog, prog) == 0) {
			/* Hay que eliminar esta entrada, coincide por la interfaz o nombre de programa */
			printf ("/// Eliminando entrada %s por via resolvconf. Prog = «%s»\n", entry->value, prog);
			free (entry);
			
			handle->resolver_entries = g_list_delete_link (handle->resolver_entries, g);
		}
		
		g = next;
	}
}

void resolv_manager_read_local_etc_resolv (NetworkInadorHandle *handle) {
	FILE *fd;
	GList *pos, *next;
	ResolvConfEntry *entry;
	
	fd = fopen ("/etc/resolv.conf", "r");
	if (fd == NULL) return;
	
	printf ("/// Ejecutando lectura de resolv.conf\n");
	/* Etiquetar las entradas con origen del archivo como "ya no en el archivo", para que cuando se lea el archivo, nos queden las que vamos a eliminar */
	/* Borrar cuáles están en el archivo */
	pos = handle->resolver_entries;
	while (pos != NULL) {
		entry = (ResolvConfEntry *) pos->data;
		
		entry->is_on_file = 0;
		pos = pos->next;
	}
	
	handle->resolver_entries = resolv_parser_parse_file (handle->resolver_entries, fd, RESOLV_ORIGIN_FILE);
	
	fclose (fd);
	
	/* A la salida, borrar las que ya no siguen en el archivo */
	pos = handle->resolver_entries;
	while (pos != NULL) {
		next = pos->next;
		entry = (ResolvConfEntry *) pos->data;
		
		if (entry->origin == RESOLV_ORIGIN_FILE && entry->is_on_file == 0) {
			/* Esta entrada se va, la eliminaron */
			printf ("/// La entrada %s del resolv.conf fué eliminada del archivo. Purgando nosotros.\n", entry->value);
			
			handle->resolver_entries = g_list_delete_link (handle->resolver_entries, pos);
			free (entry);
			entry = NULL;
		} else {
			printf ("/// La entrada %s del resolv.conf sigue en el archivo.\n", entry->value);
		}
		pos = next;
	}
}

void resolv_manager_init (NetworkInadorHandle *handle) {
	
	//handle->resolver_inotify_fd = inotify_init1 (IN_NONBLOCK | IN_CLOEXEC);
	
	if (handle->resolver_inotify_fd >= 0) {
		/* Debemos primero instalar el manejador de eventos de escritura sobre el resolv.conf */
		/*
		 IN_CLOSE_WRITE Releer archivo.
		 IN_DELETE_SELF Dejar de hacer watch
		 IN_MOVE_SELF Dejar de hacer watch
		*/
		
		//handle->resolver_inotify_watch = inotify_add_watch (handle->resolver_inotify_fd, "/etc/resolv.conf", 
	}
	 
	handle->resolver_entries = NULL;
	/* Luego, leer el resolv.conf */
	resolv_manager_read_local_etc_resolv (handle);
	
	resolv_manager_read_local_etc_resolv (handle);
	
	resolv_manager_read_local_etc_resolv (handle);
	
	resolv_manager_read_local_etc_resolv (handle);
	
	resolv_manager_read_local_etc_resolv (handle);
}

